//*****************************************************************************
// AvrDCF77_
//*****************************************************************************
// Hardware:
// ****************************************************************************
// Pollin-Eval-Board ATMega644P
// Pollin Addon-Board
// MAX7219
// 7Seg_CC x 8 digits
// ELV DCF-Empfangsmodul DCF-2
//*****************************************************************************
// DCF77 signal input @ PA7
// debug led          @ PA4
//*****************************************************************************

// uses:
// ****************************************************************************
// Library  Mcucpp (c) Konstantin Chizhov
// Mcucpp:  https://github.com/KonstantinChizhov/Mcucpp
//*****************************************************************************

// based on:
// ****************************************************************************
// dcf77	https://www.mikrocontroller.net/topic/58769#new  (c) Peter Dannegger
//*****************************************************************************

// tutorial...templates:
// ****************************************************************************
// http://www.codeproject.com/Articles/257589/An-Idiots-Guide-to-Cplusplus-Templates-Part
// http://www.codeproject.com/Articles/268849/An-Idiots-Guide-to-Cplusplus-Templates-Part
//
//*****************************************************************************

// compiled: AtmelStudio 6.2: 
// ****************************************************************************
// AVR/GNU C++ Compiler
// ****************************************************************************
// Symbols      :   F_CPU=14745600UL
// Directories  : ../~/mcucpp
//                ../~/mcucpp/AVR
//                ../project-directory
// Miscellaneous: -std=c++11
//*****************************************************************************

	#include <avr/interrupt.h>               // avr

	#include <iopins.h>                      // mcucpp
	#include <pinlist.h>                     // mcucpp

	#include <cMax7219.h>                    // display driver
 
	#include <cDCF77.h>                      // radio clock
	#include <cTimebase.h>                   // precise timebase
	
	using namespace Mcucpp;
	using namespace IO;
	using namespace Timers;
//****************************************************************************

//display driver
//*******************************************
	typedef cMax7219 < Pb1,                  // DIN
										 Pb3,                  // LOAD
										 Pb2,                  // CLK
										 8,                    // digits
										 1                     // intensity
									// default               // decode
//*******************************************
	>_Display;
	
// _Display | init displayMem[]
//*******************************************
	template<>                               // template<> is mandatory
	uint8_t _Display::displayMem[] = 
//******************************************
	                    { 0,0,0,0,0,0,0,0 }; // external init is mandatory

//precise timebase 
//*******************************************
	typedef cTimeBase _TimeBase;

//precise timebase | init static vars
//*******************************************
	uint8_t _TimeBase::_t64count= 0;         // external init is mandatory
	uint8_t _TimeBase::_timeflags=0;         // ...
	
//radio clock
//*******************************************
	typedef cDCF77< Pa4,                     // debug led
									Pa7,                     // dcf77 signal input @ PA7
								 _Display,                 // display driver
								 _TimeBase                 // precise second
//*******************************************
 > _DCF77;

// _DCF77 | init static vars
//*******************************************
	template<> sTimeDate _DCF77::_newtimedate =
//*******************************************
                        { 0,0,0,0,0,0,0 }; // external init is mandatory

	template<> uint8_t _DCF77::_pulse  = 0;  // external init is mandatory
	template<> uint8_t _DCF77::_period = 0;  // ...
	template<> uint8_t _DCF77::_error  = 0;  // ...
	template<> uint8_t _DCF77::_sync   = 0;  // ...

//clock
//*******************************************
	typedef cClock<_Display,_TimeBase> _Clock;

//_Clock | init static vars
//*******************************************
	template<> sTimeDate _Clock::_timedate = 
//*******************************************
                        { 0,0,0,0,0,0,0 }; // external init is mandatory


	ISR(TIMER0_OVF_vect) {
//*******************************************
		_TimeBase::Update();                   // precise second
			_DCF77 ::Update();                   // store data for display
	}

	int main() {
//###########################################
	_TimeBase::Init();                       // precise second
	 _Display::Init();                       // display
		 _DCF77::Init();                       // radio clock

	sei();                                   // enable int

	for(;;) {
//*******************************************
	 _DCF77::scan();                         // scan data

		if( _TimeBase::_timeflags &
		    1<<_TimeBase::ONE_SECOND ) {       // second has changed
	//*****************************************
		 _TimeBase::_timeflags = 0;
		 _Clock::clock();                      // calculate time and date

			if( _DCF77::GetSync() == 0 ) {
					_Clock::displaySync();           // wait for sync
					 continue;                       // for...
			}

		 _Clock::displayTime();                // display

		} //.if
	} //.for
} //.main
